from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class SecretVersionInfo(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)

    # The createdAt property
    created_at: Optional[datetime.datetime] = None
    # The deletedAt property
    deleted_at: Optional[datetime.datetime] = None
    # The isCurrent property
    is_current: Optional[bool] = None
    # The isDestroyed property
    is_destroyed: Optional[bool] = None
    # The version property
    version: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> SecretVersionInfo:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: SecretVersionInfo
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return SecretVersionInfo()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "deletedAt": lambda n : setattr(self, 'deleted_at', n.get_datetime_value()),
            "isCurrent": lambda n : setattr(self, 'is_current', n.get_bool_value()),
            "isDestroyed": lambda n : setattr(self, 'is_destroyed', n.get_bool_value()),
            "version": lambda n : setattr(self, 'version', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_datetime_value("deletedAt", self.deleted_at)
        writer.write_bool_value("isCurrent", self.is_current)
        writer.write_bool_value("isDestroyed", self.is_destroyed)
        writer.write_int_value("version", self.version)
        writer.write_additional_data_value(self.additional_data)
    

